# Copyright (c) Alibaba, Inc. and its affiliates.
from .fcn_head import FCNHead
from .mask2former_head import Mask2FormerHead
from .uper_head import UPerHead

__all__ = ['FCNHead', 'UPerHead', 'Mask2FormerHead']
